@@ -214,7 +214,7 @@ void KAccessApp::readSettings()
   xkb->ctrls->debounce_delay = keyboardGroup.readEntry("BounceKeysDelay", 500);
 
   // gestures for enabling the other features
-  _gestures = keyboardGroup.readEntry("Gestures", true);
+  _gestures = keyboardGroup.readEntry("Gestures", false);
   if (_gestures)
       xkb->ctrls->enabled_ctrls |= XkbAccessXKeysMask;
   else
@@ -315,16 +315,21 @@ void KAccessApp::initMasks() {
 
    for (int i = 0; strcmp (modifierKeys[i].name, "") != 0; i++) {
       int mask = modifierKeys[i].mask;
-      if (mask == 0)
-         if (modifierKeys[i].keysym != 0)
-            mask = XkbKeysymToModifiers (QX11Info::display(), modifierKeys[i].keysym);
-         else if (!strcmp(modifierKeys[i].name, "Win"))
-            mask = KKeyServer::modXMeta();
-         else
-            mask = XkbKeysymToModifiers (QX11Info::display(), XK_Mode_switch)
-                 | XkbKeysymToModifiers (QX11Info::display(), XK_ISO_Level3_Shift)
-                 | XkbKeysymToModifiers (QX11Info::display(), XK_ISO_Level3_Latch)
-                 | XkbKeysymToModifiers (QX11Info::display(), XK_ISO_Level3_Lock);
+      if (mask == 0) {
+          if (modifierKeys[i].keysym != 0) {
+              mask = XkbKeysymToModifiers (QX11Info::display(), modifierKeys[i].keysym); 
+          } else {
+              if (!strcmp(modifierKeys[i].name, "Win")) {
+                  mask = KKeyServer::modXMeta(); 
+              } else {
+                  mask = XkbKeysymToModifiers (QX11Info::display(), XK_Mode_switch)
+                      | XkbKeysymToModifiers (QX11Info::display(), XK_ISO_Level3_Shift)
+                      | XkbKeysymToModifiers (QX11Info::display(), XK_ISO_Level3_Latch)
+                      | XkbKeysymToModifiers (QX11Info::display(), XK_ISO_Level3_Lock);
+              }
+          }
+      }
+      
 
       int bit = maskToBit (mask);
       if (bit != -1 && keys[bit] == -1)
@@ -579,6 +584,7 @@ void KAccessApp::createDialogContents() {
       dialog->setButtons( KDialog::Yes | KDialog::No );
       dialog->setButtonGuiItem( KDialog::Yes, KStandardGuiItem::yes() );
       dialog->setButtonGuiItem( KDialog::No, KStandardGuiItem::no() );
+      dialog->setDefaultButton( KDialog::No );
       dialog->setEscapeButton( KDialog::Close );
       dialog->setObjectName( "AccessXWarning" );
       dialog->setModal( true );
@@ -773,7 +779,7 @@ void KAccessApp::xkbControlsNotify(XkbControlsNotifyEvent *event)
 
         createDialogContents();
         featuresLabel->setText ( question+"\n\n"+explanation
-              +" "+i18n("These AccessX settings are needed for some users with motion impairments and can be configured in the KDE Control Center. You can also turn them on and off with standardized keyboard gestures.\n\nIf you do not need them, you can select \"Deactivate all AccessX features and gestures\".") );
+              +" "+i18n("These AccessX settings are needed for some users with motion impairments and can be configured in the KDE System Settings. You can also turn them on and off with standardized keyboard gestures.\n\nIf you do not need them, you can select \"Deactivate all AccessX features and gestures\".") );
 
         KWindowSystem::setState( dialog->winId(), NET::KeepAbove );
         kapp->updateUserTimestamp();
