@@ -38,9 +38,7 @@
 
 #include <QDBusConnection>
 #include <QDBusError>
-#include <QDBusInterface>
 #include <QDBusReply>
-#include <QDBusMetaType>
 
 
 /*
@@ -197,8 +195,7 @@ void KGlobalShortcutsEditor::activateComponent(const QString &component)
 {
     QHash<QString, ComponentData*>::Iterator iter = d->components.find(component);
     if (iter == d->components.end()) {
-        kDebug() << "The component" << component << "is unknown";
-        Q_ASSERT(iter == d->components.end());
+        Q_ASSERT(iter != d->components.end());
         return;
     } else {
         int index = d->ui.components->findText(component);
@@ -440,7 +437,6 @@ void KGlobalShortcutsEditor::importConfiguration(KConfigBase *config)
     // Now import the new configurations.
     Q_FOREACH (ComponentData *cd, d->components) {
         KConfigGroup group(config, cd->uniqueName());
-        kDebug();
         if (group.exists()) {
             kDebug() << "Importing" << cd->uniqueName();
             cd->editor()->importConfiguration(&group);
