@@ -3,7 +3,11 @@
  * Copyright (C) 2002 Karol Szwed <gallium@kde.org>
  * Copyright (C) 2002 Daniel Molkentin <molkentin@kde.org>
  * Copyright (C) 2007 Urs Wolfer <uwolfer @ kde.org>
- *
+ * Copyright (C) 2009 by Davide Bettio <davide.bettio@kdemail.net>
+ 
+ * Portions Copyright (C) 2007 Paolo Capriotti <p.capriotti@gmail.com>
+ * Portions Copyright (C) 2007 Ivan Cukic <ivan.cukic+kde@gmail.com>
+ * Portions Copyright (C) 2008 by Petri Damsten <damu@iki.fi>
  * Portions Copyright (C) 2000 TrollTech AS.
  *
  * This program is free software; you can redistribute it and/or
@@ -36,14 +40,25 @@
 #include <kmessagebox.h>
 #include <kstyle.h>
 #include <kstandarddirs.h>
+#include <kautostart.h>
 #include <KDebug>
+#include <KColorScheme>
+#include <KStandardDirs>
+#include <KNS/Engine>
+
+#include <Plasma/FrameSvg>
+#include <Plasma/Theme>
 
 #include <QtCore/QFile>
 #include <QtCore/QSettings>
+#include <QtGui/QAbstractItemView>
 #include <QtGui/QLabel>
+#include <QtGui/QPainter>
 #include <QtGui/QPixmapCache>
 #include <QtGui/QStyleFactory>
 #include <QtGui/QFormLayout>
+#include <QtGui/QStandardItemModel>
+#include <QtGui/QStyle>
 #include <QtDBus/QtDBus>
 
 #ifdef Q_WS_X11
@@ -106,6 +121,181 @@ extern "C"
     }
 }
 
+class ThemeInfo
+{
+public:
+    QString package;
+    Plasma::FrameSvg *svg;
+};
+
+class ThemeModel : public QAbstractListModel
+{
+public:
+    enum { PackageNameRole = Qt::UserRole,
+           SvgRole = Qt::UserRole + 1
+         };
+
+    ThemeModel(QObject *parent = 0);
+    virtual ~ThemeModel();
+
+    virtual int rowCount(const QModelIndex &parent = QModelIndex()) const;
+    virtual QVariant data(const QModelIndex &index, int role = Qt::DisplayRole) const;
+    QModelIndex indexOf(const QString &path) const;
+    void reload();
+private:
+    QMap<QString, ThemeInfo> m_themes;
+};
+
+ThemeModel::ThemeModel( QObject *parent )
+    : QAbstractListModel( parent )
+{
+}
+
+ThemeModel::~ThemeModel()
+{
+}
+
+void ThemeModel::reload()
+{
+    reset();
+    foreach (const ThemeInfo &info, m_themes) {
+        delete info.svg;
+    }
+    m_themes.clear();
+
+    // get all desktop themes
+    KPluginInfo::List themeInfos = Plasma::Theme::listThemeInfo();
+
+    foreach (const KPluginInfo &themeInfo, themeInfos) {
+        kDebug() << themeInfo.name() << themeInfo.pluginName();
+        QString name = themeInfo.name();
+        if (name.isEmpty()) {
+            name = themeInfo.pluginName();
+        }
+
+        Plasma::Theme *theme = new Plasma::Theme(themeInfo.pluginName(), this);
+        Plasma::FrameSvg *svg = new Plasma::FrameSvg(theme);
+        svg->setUsingRenderingCache(false);
+        svg->setTheme(theme);
+        svg->setImagePath("widgets/background");
+        svg->setEnabledBorders(Plasma::FrameSvg::AllBorders);
+
+        ThemeInfo info;
+        info.package = themeInfo.pluginName();
+        info.svg = svg;
+        m_themes[name] = info;
+    }
+
+    beginInsertRows(QModelIndex(), 0, m_themes.size());
+    endInsertRows();
+}
+
+int ThemeModel::rowCount(const QModelIndex &) const
+{
+    return m_themes.size();
+}
+
+QVariant ThemeModel::data(const QModelIndex &index, int role) const
+{
+    if (!index.isValid()) {
+        return QVariant();
+    }
+
+    if (index.row() >= m_themes.size()) {
+        return QVariant();
+    }
+
+    QMap<QString, ThemeInfo>::const_iterator it = m_themes.constBegin();
+    for (int i = 0; i < index.row(); ++i) {
+        ++it;
+    }
+
+    switch (role) {
+        case Qt::DisplayRole:
+            return it.key();
+        case PackageNameRole:
+            return (*it).package;
+        case SvgRole:
+            return qVariantFromValue((void*)(*it).svg);
+        default:
+            return QVariant();
+    }
+}
+
+QModelIndex ThemeModel::indexOf(const QString &name) const
+{
+    QMapIterator<QString, ThemeInfo> it(m_themes);
+    int i = -1;
+    while (it.hasNext()) {
+        ++i;
+        if (it.next().value().package == name) {
+            return index(i, 0);
+        }
+    }
+
+    return QModelIndex();
+}
+
+
+class ThemeDelegate : public QAbstractItemDelegate
+{
+public:
+    ThemeDelegate(QObject * parent = 0);
+
+    virtual void paint(QPainter *painter,
+                       const QStyleOptionViewItem &option,
+                       const QModelIndex &index) const;
+    virtual QSize sizeHint(const QStyleOptionViewItem &option,
+                           const QModelIndex &index) const;
+private:
+    static const int MARGIN = 5;
+};
+
+ThemeDelegate::ThemeDelegate(QObject* parent)
+: QAbstractItemDelegate(parent)
+{
+}
+
+void ThemeDelegate::paint(QPainter *painter,
+                          const QStyleOptionViewItem &option,
+                          const QModelIndex &index) const
+{
+    QString title = index.model()->data(index, Qt::DisplayRole).toString();
+    QString package = index.model()->data(index, ThemeModel::PackageNameRole).toString();
+
+    // highlight selected item
+    QApplication::style()->drawControl(QStyle::CE_ItemViewItem, &option, painter);
+
+    // draw image
+    Plasma::FrameSvg *svg = static_cast<Plasma::FrameSvg *>(
+            index.model()->data(index, ThemeModel::SvgRole).value<void *>());
+    svg->resizeFrame(QSize(option.rect.width() - (2 * MARGIN), 100 - (2 * MARGIN)));
+    QRect imgRect = QRect(option.rect.topLeft(),
+            QSize(option.rect.width() - (2 * MARGIN), 100 - (2 * MARGIN)))
+            .translated(MARGIN, MARGIN);
+    svg->paintFrame(painter, QPoint(option.rect.left() + MARGIN, option.rect.top() + MARGIN));
+
+    // draw text
+    painter->save();
+    QFont font = painter->font();
+    font.setWeight(QFont::Bold);
+    QString colorFile = KStandardDirs::locate("data", "desktoptheme/" + package + "/colors");
+    if (!colorFile.isEmpty()) {
+        KSharedConfigPtr colors = KSharedConfig::openConfig(colorFile);
+        KColorScheme colorScheme(QPalette::Active, KColorScheme::Window, colors);
+        painter->setPen(colorScheme.foreground(KColorScheme::NormalText).color());
+    }
+    painter->setFont(font);
+    painter->drawText(option.rect, Qt::AlignCenter | Qt::TextWordWrap, title);
+    painter->restore();
+}
+
+
+QSize ThemeDelegate::sizeHint(const QStyleOptionViewItem &, const QModelIndex &) const
+{
+    return QSize(200, 100);
+}
+
 class StylePreview : public QWidget, public Ui::StylePreview
 {
 public:
@@ -155,9 +345,14 @@ KCMStyle::KCMStyle( QWidget* parent, const QVariantList& )
 			"of user interface elements, such as the widget style "
 			"and effects."));
 
+	m_bDesktopThemeDirty = false;
 	m_bStyleDirty= false;
 	m_bEffectsDirty = false;
 
+
+	KAutostart plasmaNetbookAutoStart("plasma-netbook");
+	m_isNetbook = plasmaNetbookAutoStart.autostarts();
+
 	KGlobal::dirs()->addResourceType("themes", "data", "kstyle/themes");
 
 	KAboutData *about =
@@ -178,12 +373,29 @@ KCMStyle::KCMStyle( QWidget* parent, const QVariantList& )
 	tabWidget  = new QTabWidget( this );
 	mainLayout->addWidget( tabWidget );
 
-	page1 = new QWidget;
-	page1Layout = new QVBoxLayout( page1 );
+	// Add Page0 (Desktop Theme)
+	// -------------------
+	page0 = new QWidget;
+	themeUi.setupUi(page0);
 
-	// Add Page1 (Style)
+	themeUi.m_newThemeButton->setIcon(KIcon("get-hot-new-stuff"));
+	
+	m_themeModel = new ThemeModel(this);
+	themeUi.m_theme->setModel(m_themeModel);
+	themeUi.m_theme->setItemDelegate(new ThemeDelegate(themeUi.m_theme));
+	themeUi.m_theme->setVerticalScrollMode(QAbstractItemView::ScrollPerPixel);
+
+	connect(themeUi.m_theme, SIGNAL(clicked(const QModelIndex &)), this, SLOT(setDesktopThemeDirty()));
+	connect(themeUi.m_newThemeButton, SIGNAL(clicked()), this, SLOT(getNewThemes()));
+
+	m_workspaceThemeTabActivated = false;
+  
+	// Add Page1 (Applications Style)
 	// -----------------
 	//gbWidgetStyle = new QGroupBox( i18n("Widget Style"), page1 );
+	page1 = new QWidget;
+	page1Layout = new QVBoxLayout( page1 );
+	
 	QWidget* gbWidgetStyle = new QWidget( page1 );
 	QVBoxLayout *widgetLayout = new QVBoxLayout(gbWidgetStyle);
 
@@ -229,43 +441,31 @@ KCMStyle::KCMStyle( QWidget* parent, const QVariantList& )
 	// Add Page2 (Effects)
 	// -------------------
 	page2 = new QWidget;
-	QFormLayout* page2Layout = new QFormLayout( page2 );
+	fineTuningUi.setupUi(page2);
 
-	comboGraphicEffectsLevel = new KComboBox( page2 );
-	comboGraphicEffectsLevel->setVisible(false);
-	comboGraphicEffectsLevel->setObjectName( "cbGraphicEffectsLevel" );
-	comboGraphicEffectsLevel->setEditable( false );
-	comboGraphicEffectsLevel->addItem(i18n("Low display resolution and Low CPU"), KGlobalSettings::NoEffects);
-	comboGraphicEffectsLevel->addItem(i18n("High display resolution and Low CPU"), KGlobalSettings::GradientEffects);
-	comboGraphicEffectsLevel->addItem(i18n("Low display resolution and High CPU"), KGlobalSettings::SimpleAnimationEffects);
-	comboGraphicEffectsLevel->addItem(i18n("High display resolution and High CPU"), (int) (KGlobalSettings::SimpleAnimationEffects | KGlobalSettings::GradientEffects));
-	comboGraphicEffectsLevel->addItem(i18n("Low display resolution and Very High CPU"), KGlobalSettings::ComplexAnimationEffects);
-	comboGraphicEffectsLevel->addItem(i18n("High display resolution and Very High CPU"), (int) (KGlobalSettings::ComplexAnimationEffects | KGlobalSettings::GradientEffects));
-	//page2Layout->addRow(i18nc("@label:listbox","Graphical User Interface:"), comboGraphicEffectsLevel);
-
-	comboToolbarIcons = new QComboBox( page2 );
-	comboToolbarIcons->setEditable( false );
-	comboToolbarIcons->addItem( i18n("Icons Only") );
-	comboToolbarIcons->addItem( i18n("Text Only") );
-	comboToolbarIcons->addItem( i18n("Text Alongside Icons") );
-	comboToolbarIcons->addItem( i18n("Text Under Icons") );
-	comboToolbarIcons->setSizePolicy( QSizePolicy::Minimum, QSizePolicy::Minimum );
-	page2Layout->addRow(i18nc("@label:listbox","Text pos&ition of toolbar elements:"), comboToolbarIcons);
-
-	cbIconsOnButtons = new QCheckBox( i18nc("@option:check","Sho&w icons on buttons"), page2 );
- 	page2Layout->addRow(cbIconsOnButtons);
+	fineTuningUi.comboGraphicEffectsLevel->setObjectName( "cbGraphicEffectsLevel" );
+	fineTuningUi.comboGraphicEffectsLevel->setEditable( false );
+	fineTuningUi.comboGraphicEffectsLevel->addItem(i18n("Low display resolution and Low CPU"), KGlobalSettings::NoEffects);
+	fineTuningUi.comboGraphicEffectsLevel->addItem(i18n("High display resolution and Low CPU"), KGlobalSettings::GradientEffects);
+	fineTuningUi.comboGraphicEffectsLevel->addItem(i18n("Low display resolution and High CPU"), KGlobalSettings::SimpleAnimationEffects);
+	fineTuningUi.comboGraphicEffectsLevel->addItem(i18n("High display resolution and High CPU"), (int) (KGlobalSettings::SimpleAnimationEffects | KGlobalSettings::GradientEffects));
+	fineTuningUi.comboGraphicEffectsLevel->addItem(i18n("Low display resolution and Very High CPU"), KGlobalSettings::ComplexAnimationEffects);
+	fineTuningUi.comboGraphicEffectsLevel->addItem(i18n("High display resolution and Very High CPU"), (int) (KGlobalSettings::ComplexAnimationEffects | KGlobalSettings::GradientEffects));
 
 	connect(cbStyle, SIGNAL(activated(int)), this, SLOT(setStyleDirty()));
-	connect( cbIconsOnButtons,     SIGNAL(toggled(bool)),   this, SLOT(setEffectsDirty()));
-	connect( comboGraphicEffectsLevel, SIGNAL(activated(int)),   this, SLOT(setEffectsDirty()));
-	connect( comboToolbarIcons,    SIGNAL(activated(int)), this, SLOT(setEffectsDirty()));
+	connect(fineTuningUi.cbIconsOnButtons,     SIGNAL(toggled(bool)),   this, SLOT(setEffectsDirty()));
+	connect(fineTuningUi.comboGraphicEffectsLevel, SIGNAL(activated(int)),   this, SLOT(setEffectsDirty()));
+	connect(fineTuningUi.comboToolbarIcons,    SIGNAL(activated(int)), this, SLOT(setEffectsDirty()));
+	connect(fineTuningUi.comboSecondaryToolbarIcons,    SIGNAL(activated(int)), this, SLOT(setEffectsDirty()));
 
 	addWhatsThis();
 
 	// Insert the pages into the tabWidget
-	tabWidget->addTab( page1, i18nc("@title:tab","&Style"));
-	tabWidget->addTab( page2, i18nc("@title:tab","&Fine Tuning"));
+	tabWidget->addTab(page1, i18nc("@title:tab", "&Applications"));
+	tabWidget->addTab(page0, i18nc("@title:tab", "&Workspace"));
+	tabWidget->addTab(page2, i18nc("@title:tab", "&Fine Tuning"));
 
+	connect(tabWidget, SIGNAL(currentChanged(int)), this, SLOT(tabChanged(int)));
 }
 
 
@@ -360,9 +560,13 @@ void KCMStyle::load()
 {
 	KConfig config( "kdeglobals", KConfig::FullConfig );
 
+	if (m_workspaceThemeTabActivated) {
+		loadDesktopTheme();
+	}
 	loadStyle( config );
 	loadEffects( config );
 
+	m_bDesktopThemeDirty = false;
 	m_bStyleDirty= false;
 	m_bEffectsDirty = false;
 	//Enable/disable the button for the initial style
@@ -375,31 +579,37 @@ void KCMStyle::load()
 void KCMStyle::save()
 {
 	// Don't do anything if we don't need to.
-	if ( !( m_bStyleDirty | m_bEffectsDirty ) )
+	if ( !( m_bDesktopThemeDirty | m_bStyleDirty | m_bEffectsDirty ) )
 		return;
 
+	//Desktop theme
+	if ( m_bDesktopThemeDirty )
+	{
+		QString theme = m_themeModel->data(themeUi.m_theme->currentIndex(), ThemeModel::PackageNameRole).toString();
+		if (m_isNetbook) {
+			KConfigGroup cg(KSharedConfig::openConfig("plasmarc"), "Theme-plasma-netbook");
+			cg.writeEntry("name", theme);
+		} else {
+			Plasma::Theme::defaultTheme()->setThemeName(theme);
+		}
+	}
+
 	// Save effects.
         KConfig      _config("kdeglobals", KConfig::NoGlobals);                
         KConfigGroup config(&_config, "KDE"); 
 	// Effects page
-	config.writeEntry( "ShowIconsOnPushButtons", cbIconsOnButtons->isChecked());
+	config.writeEntry( "ShowIconsOnPushButtons", fineTuningUi.cbIconsOnButtons->isChecked());
 	KConfigGroup g( &_config, "KDE-Global GUI Settings" );
-	g.writeEntry( "GraphicEffectsLevel", comboGraphicEffectsLevel->itemData(comboGraphicEffectsLevel->currentIndex()));
+	g.writeEntry( "GraphicEffectsLevel", fineTuningUi.comboGraphicEffectsLevel->itemData(fineTuningUi.comboGraphicEffectsLevel->currentIndex()));
 
 	KConfigGroup generalGroup(&_config, "General");
 	generalGroup.writeEntry("widgetStyle", currentStyle());
 
 	KConfigGroup toolbarStyleGroup(&_config, "Toolbar style");
-	QString tbIcon;
-	switch( comboToolbarIcons->currentIndex() )
-	{
-		case 0: tbIcon = "IconOnly"; break;
-		case 1: tbIcon = "TextOnly"; break;
-		case 2: tbIcon = "TextBesideIcon"; break;
-		default:
-		case 3: tbIcon = "TextUnderIcon"; break;
-	}
-	toolbarStyleGroup.writeEntry("ToolButtonStyle", tbIcon);
+	toolbarStyleGroup.writeEntry("ToolButtonStyle",
+							toolbarButtonText(fineTuningUi.comboToolbarIcons->currentIndex()));
+	toolbarStyleGroup.writeEntry("ToolButtonStyleOtherToolbars",
+							toolbarButtonText(fineTuningUi.comboSecondaryToolbarIcons->currentIndex()));
 	_config.sync();
 
 	// Export the changes we made to qtrc, and update all qt-only
@@ -435,6 +645,7 @@ void KCMStyle::save()
 	}
 
 	// Clean up
+	m_bDesktopThemeDirty    = false;
 	m_bStyleDirty    = false;
 	m_bEffectsDirty  = false;
 	emit changed( false );
@@ -485,9 +696,16 @@ void KCMStyle::defaults()
 	switchStyle( currentStyle() );	// make resets visible
 
 	// Effects
-	comboToolbarIcons->setCurrentIndex(0);
-	cbIconsOnButtons->setChecked(true);
-	comboGraphicEffectsLevel->setCurrentIndex(comboGraphicEffectsLevel->findData(((int) KGlobalSettings::graphicEffectsLevelDefault())));
+	fineTuningUi.comboToolbarIcons->setCurrentIndex(toolbarButtonIndex("TextBesideIcon"));
+	fineTuningUi.comboSecondaryToolbarIcons->setCurrentIndex(toolbarButtonIndex("TextBesideIcon"));
+	fineTuningUi.cbIconsOnButtons->setChecked(true);
+	fineTuningUi.comboGraphicEffectsLevel->setCurrentIndex(fineTuningUi.comboGraphicEffectsLevel->findData(((int) KGlobalSettings::graphicEffectsLevelDefault())));
+	emit changed(true);
+}
+
+void KCMStyle::setDesktopThemeDirty()
+{
+	m_bDesktopThemeDirty = true;
 	emit changed(true);
 }
 
@@ -503,6 +721,45 @@ void KCMStyle::setStyleDirty()
 	emit changed(true);
 }
 
+void KCMStyle::tabChanged(int index)
+{
+	if (index == 1 && !m_workspaceThemeTabActivated) { //Workspace theme tab (never loaded before)
+		m_workspaceThemeTabActivated = true;
+		QTimer::singleShot(100, this, SLOT(loadDesktopTheme()));
+	}
+}
+
+// ----------------------------------------------------------------
+// All the Desktop Theme stuff
+// ----------------------------------------------------------------
+
+void KCMStyle::getNewThemes()
+{
+	KNS::Engine engine(this);
+	if (engine.init("plasma-themes.knsrc")) {
+		KNS::Entry::List entries = engine.downloadDialogModal(this);
+
+		if (entries.size() > 0) {
+			loadDesktopTheme();
+		}
+	}
+}
+
+void KCMStyle::loadDesktopTheme()
+{
+	QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
+	m_themeModel->reload();
+	QString themeName;
+	if (m_isNetbook) {
+		KConfigGroup cg(KSharedConfig::openConfig("plasmarc"), "Theme-plasma-netbook");
+		themeName = cg.readEntry("name", "air-netbook");
+	} else {
+		themeName = Plasma::Theme::defaultTheme()->themeName();
+	}
+	themeUi.m_theme->setCurrentIndex(m_themeModel->indexOf(themeName));
+	QApplication::restoreOverrideCursor();
+}
+
 // ----------------------------------------------------------------
 // All the Style Switching / Preview stuff
 // ----------------------------------------------------------------
@@ -677,27 +934,50 @@ void KCMStyle::setStyleRecursive(QWidget* w, QStyle* s)
 // ----------------------------------------------------------------
 // All the Effects stuff
 // ----------------------------------------------------------------
+QString KCMStyle::toolbarButtonText(int index)
+{
+	switch (index) {
+		case 1:
+			return "TextOnly";
+		case 2:
+			return "TextBesideIcon";
+		case 3:
+			return "TextUnderIcon";
+		default:
+			break;
+	}
+
+	return "NoText";
+}
+
+int KCMStyle::toolbarButtonIndex(const QString &text)
+{
+	if (text == "TextOnly") {
+		return 1;
+	} else if (text == "TextBesideIcon") {
+		return 2;
+	} else if (text == "TextUnderIcon") {
+		return 3;
+	}
+
+	return 0;
+}
 
 void KCMStyle::loadEffects( KConfig& config )
 {
 	// KDE's Part via KConfig
 	KConfigGroup configGroup = config.group("Toolbar style");
 
-	QString tbIcon = configGroup.readEntry("ToolButtonStyle", "TextUnderIcon");
-	if (tbIcon == "TextOnly")
-		comboToolbarIcons->setCurrentIndex(1);
-	else if (tbIcon == "TextBesideIcon")
-		comboToolbarIcons->setCurrentIndex(2);
-	else if (tbIcon == "TextUnderIcon")
-		comboToolbarIcons->setCurrentIndex(3);
-	else
-		comboToolbarIcons->setCurrentIndex(0);
+	QString tbIcon = configGroup.readEntry("ToolButtonStyle", "TextBesideIcon");
+	fineTuningUi.comboToolbarIcons->setCurrentIndex(toolbarButtonIndex(tbIcon));
+	tbIcon = configGroup.readEntry("ToolButtonStyleOtherToolbars", "TextBesideIcon");
+	fineTuningUi.comboSecondaryToolbarIcons->setCurrentIndex(toolbarButtonIndex(tbIcon));
 
 	configGroup = config.group("KDE");
-	cbIconsOnButtons->setChecked(configGroup.readEntry("ShowIconsOnPushButtons", true));
+	fineTuningUi.cbIconsOnButtons->setChecked(configGroup.readEntry("ShowIconsOnPushButtons", true));
 
 	KConfigGroup graphicConfigGroup = config.group("KDE-Global GUI Settings");
-	comboGraphicEffectsLevel->setCurrentIndex(comboGraphicEffectsLevel->findData(graphicConfigGroup.readEntry("GraphicEffectsLevel", ((int) KGlobalSettings::graphicEffectsLevel()))));
+	fineTuningUi.comboGraphicEffectsLevel->setCurrentIndex(fineTuningUi.comboGraphicEffectsLevel->findData(graphicConfigGroup.readEntry("GraphicEffectsLevel", ((int) KGlobalSettings::graphicEffectsLevel()))));
 
 	m_bEffectsDirty = false;
 }
@@ -713,16 +993,16 @@ void KCMStyle::addWhatsThis()
 							"without having to apply it to the whole desktop.") );
 	// Page2
 	page2->setWhatsThis( i18n("This page allows you to choose details about the widget style options") );
-	comboToolbarIcons->setWhatsThis( i18n( "<p><b>Icons only:</b> Shows only icons on toolbar buttons. "
+	fineTuningUi.comboToolbarIcons->setWhatsThis( i18n( "<p><b>No Text:</b> Shows only icons on toolbar buttons. "
 							"Best option for low resolutions.</p>"
-							"<p><b>Text only: </b>Shows only text on toolbar buttons.</p>"
-							"<p><b>Text alongside icons: </b> Shows icons and text on toolbar buttons. "
-							"Text is aligned alongside the icon.</p>"
-							"<b>Text under icons: </b> Shows icons and text on toolbar buttons. "
+							"<p><b>Text Only: </b>Shows only text on toolbar buttons.</p>"
+							"<p><b>Text Beside Icons: </b> Shows icons and text on toolbar buttons. "
+							"Text is aligned beside the icon.</p>"
+							"<b>Text Below Icons: </b> Shows icons and text on toolbar buttons. "
 							"Text is aligned below the icon.") );
-	cbIconsOnButtons->setWhatsThis( i18n( "If you enable this option, KDE Applications will "
+	fineTuningUi.cbIconsOnButtons->setWhatsThis( i18n( "If you enable this option, KDE Applications will "
 							"show small icons alongside some important buttons.") );
-	comboGraphicEffectsLevel->setWhatsThis( i18n( "If you enable this option, KDE Applications will "
+	fineTuningUi.comboGraphicEffectsLevel->setWhatsThis( i18n( "If you enable this option, KDE Applications will "
 							"run internal animations.") );
 }
 
