@@ -52,6 +52,7 @@ from the copyright holder.
 
 static void sigHandler( int n );
 static int scanConfigs( int force );
+static void startDisplay( struct display *d );
 static void startDisplays( void );
 #define XS_KEEP 0
 #define XS_RESTART 1
@@ -296,10 +297,10 @@ main( int argc, char **argv )
 	/*
 	 * Step 2 - run a sub-daemon for each entry
 	 */
+	openCtrl( 0 );
 #ifdef XDMCP
 	updateListenSockets();
 #endif
-	openCtrl( 0 );
 	mainLoop();
 	closeCtrl( 0 );
 	if (sdRec.how) {
@@ -352,11 +353,15 @@ wakeDisplay( struct display *d )
 }
 #endif
 
-enum utState { UtDead, UtWait, UtActive };
+enum utState {
+	UtDead,   /* no such entry */
+	UtWait,   /* waiting for user login */
+	UtActive  /* user logged in */
+};
 
 struct utmps {
-	struct utmps *next;
 #ifndef HAVE_VTS
+	struct utmps *next;
 	struct display *d;
 #endif
 	time_t time;
@@ -370,20 +375,21 @@ struct utmps {
 static struct utmps *utmpList;
 static time_t utmpTimeout = TO_INF;
 
-static void
-bombUtmp( void )
+void
+wakeDisplays( void )
 {
-	struct utmps *utp;
-
-	while ((utp = utmpList)) {
 #ifdef HAVE_VTS
-		forEachDisplay( wakeDisplay );
+	forEachDisplay( wakeDisplay );
+	free( utmpList );
+	utmpList = 0;
 #else
+	struct utmps *utp;
+	while ((utp = utmpList)) {
 		utp->d->status = notRunning;
-#endif
 		utmpList = utp->next;
 		free( utp );
 	}
+#endif
 }
 
 static void
@@ -392,7 +398,10 @@ checkUtmp( void )
 	static time_t modtim;
 	time_t nck;
 	time_t ends;
-	struct utmps *utp, **utpp;
+	struct utmps *utp;
+#ifndef HAVE_VTS
+	struct utmps **utpp;
+#endif
 	struct stat st;
 #ifdef BSD_UTMP
 	int fd;
@@ -405,17 +414,21 @@ checkUtmp( void )
 		return;
 	if (stat( UTMP_FILE, &st )) {
 		logError( UTMP_FILE " not found - cannot use console mode\n" );
-		bombUtmp();
+		wakeDisplays();
 		return;
 	}
 	if (modtim != st.st_mtime) {
 		debug( "rescanning " UTMP_FILE "\n" );
+#ifdef HAVE_VTS
+		utp = utmpList;
+#else
 		for (utp = utmpList; utp; utp = utp->next)
+#endif
 			utp->state = UtDead;
 #ifdef BSD_UTMP
 		if ((fd = open( UTMP_FILE, O_RDONLY )) < 0) {
 			logError( "Cannot open " UTMP_FILE " - cannot use console mode\n" );
-			bombUtmp();
+			wakeDisplays();
 			return;
 		}
 		while (reader( fd, ut, sizeof(ut[0]) ) == sizeof(ut[0]))
@@ -424,36 +437,39 @@ checkUtmp( void )
 		while ((ut = GETUTENT()))
 #endif
 		{
-			for (utp = utmpList; utp; utp = utp->next) {
+			/* first, match the tty to a utemps */
 #ifdef HAVE_VTS
-				char **line;
-				for (line = consoleTTYs; *line; line++)
-					if (!strncmp( *line, ut->ut_line, sizeof(ut->ut_line) ))
-						goto hitlin;
-				continue;
-			  hitlin:
+			char **line;
+			for (line = consoleTTYs; *line; line++)
+				if (!strncmp( *line, ut->ut_line, sizeof(ut->ut_line) ))
 #else
-				if (strncmp( utp->d->console, ut->ut_line, sizeof(ut->ut_line) ))
+			for (utp = utmpList; utp; utp = utp->next)
+				if (!strncmp( utp->d->console, ut->ut_line, sizeof(ut->ut_line) ))
+#endif
+					goto hitlin;
+			continue;
+		  hitlin:
+			/* then, update the utemps accordingly */
+#ifdef BSD_UTMP
+			if (!*ut->ut_user) {
+#else
+			if (ut->ut_type != USER_PROCESS) {
+#endif
+#ifdef HAVE_VTS
+				/* don't allow "downgrading" the singular utemps */
+				if (utp->state == UtActive)
 					continue;
 #endif
-#ifdef BSD_UTMP
-				if (!*ut->ut_user) {
-#else
-				if (ut->ut_type != USER_PROCESS) {
-#endif
-#ifdef HAVE_VTS
-					if (utp->state == UtActive)
-						break;
-#endif
-					utp->state = UtWait;
-				} else {
-					utp->hadSess = True;
-					utp->state = UtActive;
-				}
-				if (utp->time < ut->ut_time) /* theoretically superfluous */
-					utp->time = ut->ut_time;
-				break;
+				utp->state = UtWait;
+			} else {
+				utp->hadSess = True;
+				utp->state = UtActive;
 			}
+#ifdef HAVE_VTS
+			/* tty with latest activity wins */
+			if (utp->time < ut->ut_time)
+#endif
+				utp->time = ut->ut_time;
 		}
 #ifdef BSD_UTMP
 		close( fd );
@@ -462,31 +478,38 @@ checkUtmp( void )
 #endif
 		modtim = st.st_mtime;
 	}
+#ifdef HAVE_VTS
+	utp = utmpList;
+#else
 	for (utpp = &utmpList; (utp = *utpp); ) {
+#endif
 		if (utp->state != UtActive) {
 			if (utp->state == UtDead) /* shouldn't happen ... */
 				utp->time = 0;
 			ends = utp->time + (utp->hadSess ? TIME_RELOG : TIME_LOG);
 			if (ends <= now) {
 #ifdef HAVE_VTS
-				forEachDisplay( wakeDisplay );
+				wakeDisplays();
 				debug( "console login timed out\n" );
+				return;
 #else
 				utp->d->status = notRunning;
 				debug( "console login for %s at %s timed out\n",
 				       utp->d->name, utp->d->console );
-#endif
 				*utpp = utp->next;
 				free( utp );
 				continue;
+#endif
 			} else
 				nck = ends;
 		} else
 			nck = TIME_RELOG + now;
 		if (nck < utmpTimeout)
 			utmpTimeout = nck;
+#ifndef HAVE_VTS
 		utpp = &(*utpp)->next;
 	}
+#endif
 }
 
 static void
@@ -503,7 +526,7 @@ switchToTTY( struct display *d )
 
 	if (!(utp = Malloc( sizeof(*utp) ))) {
 #ifdef HAVE_VTS
-		forEachDisplay( wakeDisplay );
+		wakeDisplays();
 #else
 		d->status = notRunning;
 #endif
@@ -512,10 +535,10 @@ switchToTTY( struct display *d )
 #ifndef HAVE_VTS
 	d->status = textMode;
 	utp->d = d;
+	utp->next = utmpList;
 #endif
 	utp->time = now;
 	utp->hadSess = False;
-	utp->next = utmpList;
 	utmpList = utp;
 	checkUtmp();
 
@@ -629,17 +652,14 @@ static void
 sessionDone( struct display *d )
 {
 	d->userSess = -1;
-	if (d->userName)
-		free( d->userName );
-	d->userName = 0;
-	if (d->sessName)
-		free( d->sessName );
-	d->sessName = 0;
+	free( d->userName );
+	free( d->sessName );
+	d->userName = d->sessName = 0;
 }
 
 void
 setNLogin( struct display *d,
-           const char *nuser, const char *npass, char *nargs, int rl )
+           const char *nuser, const char *npass, const char *nargs, int rl )
 {
 	struct disphist *he = d->hstent;
 	he->rLogin =
@@ -689,10 +709,9 @@ processDPipe( struct display *d )
 				    !((d->allowNuke == SHUT_NONE && sdRec.uid != d->sdRec.uid) ||
 				      (d->allowNuke == SHUT_ROOT && d->sdRec.uid)))
 				{
-					if (sdRec.osname)
-						free( sdRec.osname );
+					free( sdRec.osname );
 					sdRec = d->sdRec;
-				} else if (d->sdRec.osname)
+				} else
 					free( d->sdRec.osname );
 				d->sdRec.how = 0;
 				d->sdRec.osname = 0;
@@ -725,8 +744,7 @@ processDPipe( struct display *d )
 		XdmcpDisposeARRAY8( &ca );
 		break;
 	case D_RemoteHost:
-		if (d->remoteHost)
-			free( d->remoteHost );
+		free( d->remoteHost );
 		d->remoteHost = gRecvStr();
 		break;
 #endif
@@ -815,8 +833,7 @@ processGPipe( struct display *d )
 		sdRec.uid = gRecvInt();
 		option = gRecvStr();
 		setBootOption( option, &sdRec );
-		if (option)
-			free( option );
+		free( option );
 		break;
 	case G_QueryShutdown:
 		gSendInt( sdRec.how );
@@ -894,10 +911,8 @@ void
 cancelShutdown( void )
 {
 	sdRec.how = 0;
-	if (sdRec.osname) {
-		free( sdRec.osname );
-		sdRec.osname = 0;
-	}
+	free( sdRec.osname );
+	sdRec.osname = 0;
 	stopping = False;
 	rescanConfigs( True );
 }
@@ -1379,7 +1394,7 @@ startDisplays( void )
 	forEachDisplay( kickDisplay );
 }
 
-void
+static void
 startDisplay( struct display *d )
 {
 	if (stopping) {
@@ -1437,6 +1452,7 @@ startDisplayP2( struct display *d )
 	               &d->gpipe, cgname, 0, &d->pid ))
 	{
 	case 0:
+		td = d;
 #ifndef NOXDMTITLE
 		setproctitle( "%s", d->name );
 #endif
@@ -1452,7 +1468,7 @@ startDisplayP2( struct display *d )
 			gSet( &mstrtalk );
 			gSendInt( D_XConnOk );
 		}
-		manageSession( d );
+		manageSession();
 		/* NOTREACHED */
 	case -1:
 		closeCtrl( d );
@@ -1551,7 +1567,7 @@ exitDisplay( struct display *d,
 			return;
 		}
 		if (endState != DS_RESTART ||
-		    (d->displayType & d_origin) != dFromFile)
+		    (d->displayType & d_lifetime) == dTransient)
 		{
 			rStopDisplay( d, endState );
 		} else {
@@ -1640,7 +1656,8 @@ storePid( void )
 #endif
 		}
 		fprintf( pidFilePtr, "%ld\n", (long)getpid() );
-		(void)fflush( pidFilePtr );
+		if (fflush( pidFilePtr ) == EOF)
+			logError( "Cannot write PID file %s: %m", pidFile );
 		registerCloseOnFork( pidFd );
 	}
 	return 0;
