@@ -35,6 +35,7 @@ from the copyright holder.
  */
 
 #include "dm.h"
+#include "dm_auth.h"
 #include "dm_error.h"
 
 #include <string.h>
@@ -186,10 +187,8 @@ strApp( char **dst, ... )
 	}
 	va_end( va );
 	if (!(bk = Malloc( len ))) {
-		if (*dst) {
-			free( *dst );
-			*dst = 0;
-		}
+		free( *dst );
+		*dst = 0;
 		return False;
 	}
 	dp = bk;
@@ -451,9 +450,7 @@ setEnv( char **e, const char *name, const char *value )
 			}
 		envsize = old - e;
 	}
-	if (!(new = (char **)
-	      Realloc( (char *)e, (unsigned)((envsize + 2) * sizeof(char *)) )))
-	{
+	if (!(new = Realloc( e, (unsigned)((envsize + 2) * sizeof(char *)) ))) {
 		free( newe );
 		return e;
 	}
@@ -468,9 +465,9 @@ putEnv( const char *string, char **env )
 	char *b, *n;
 
 	if (!(b = strchr( string, '=' )))
-		return NULL;
+		return 0;
 	if (!strNDup( &n, string, b - string ))
-		return NULL;
+		return 0;
 	env = setEnv( env, n, b + 1 );
 	free( n );
 	return env;
@@ -522,7 +519,7 @@ atomicIO( ssize_t (*f)( int, void *, size_t ), int fd, void *buf, int count )
 
 	for (rlen = 0; rlen < count; ) {
 	  dord:
-		ret = f( fd, (void *)((char *)buf + rlen), count - rlen );
+		ret = f( fd, (char *)buf + rlen, count - rlen );
 		if (ret < 0) {
 			if (errno == EINTR)
 				goto dord;
@@ -574,6 +571,20 @@ mTime( const char *fn )
 		return st.st_mtime;
 }
 
+void
+randomStr( char *s )
+{
+	static const char letters[] =
+		"abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789";
+	unsigned i, rn = secureRandom();
+
+	for (i = 0; i < 6; i++) {
+		*s++ = letters[rn % 62];
+		rn /= 62;
+	}
+	*s = 0;
+}
+
 static int
 strNChrCnt( const char *s, int slen, char c )
 {
@@ -585,6 +596,31 @@ strNChrCnt( const char *s, int slen, char c )
 	return cnt;
 }
 
+#define atox(c) ('0' <= c && c <= '9' ? c - '0' : \
+				 'a' <= c && c <= 'f' ? c - 'a' + 10 : \
+				 'A' <= c && c <= 'F' ? c - 'A' + 10 : -1)
+
+int
+hexToBinary( char *out, const char *in )
+{
+	int top, bottom;
+	char c;
+	char *oout;
+
+	for (oout = out; (c = *in++); ) {
+		if ((top = atox( c )) < 0)
+			return 0;
+		if (!(c = *in++))
+			return 0;
+		if ((bottom = atox( c )) < 0)
+			return 0;
+		*out++ = (top << 4) | bottom;
+	}
+	return out - oout;
+}
+
+#undef atox
+
 /* X -from ip6-addr does not work here, so i don't know whether this is needed.
 #define IP6_MAGIC
 */
@@ -628,6 +664,8 @@ listSessions( int flags, struct display *d, void *ctx,
 # endif
 		   )
 		{
+			if (ut->ut_pid <= 0 || (kill (ut->ut_pid, 0) < 0 && errno == ESRCH))
+				continue; /* ignore stale utmp entries */
 #endif
 			if (*ut->ut_host) { /* from remote or x */
 				if (!(flags & lstRemote))
