@@ -24,6 +24,7 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
 #include "kdmconfig.h"
 #include "kdm_greet.h"
+#include "utils.h"
 
 #include <kconfiggroup.h>
 #include <kglobal.h>
@@ -34,67 +35,42 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 #include <unistd.h>
 #include <sys/utsname.h>
 
+#define WANT_GREET_DEFS
+#include <config.ci>
+
 CONF_GREET_DEFS
 
 QString _stsFile;
 bool _isLocal;
 bool _isReserve;
 bool _authorized;
+int _grabInput;
 
 static QString
 getCfgQStr( int id )
 {
-	char *tmp = getCfgStr( id );
-	QString qs = QString::fromUtf8( tmp );
-	free( tmp );
-	return qs;
+	return qString( getCfgStr( id ) );
 }
 
 static QStringList
 getCfgQStrList( int id )
 {
-	int i, len;
-	char **tmp = getCfgStrArr( id, &len );
-	QStringList qsl;
-	for (i = 0; i < len - 1; i++) {
-		qsl.append( QString::fromUtf8( tmp[i] ) );
-		free( tmp[i] );
-	}
-	free( tmp );
-	return qsl;
+	return qStringList( getCfgStrArr( id, 0 ) );
 }
 
 // Based on kconfiggroupgui.cpp
 static QFont *
 str2Font( const QString &aValue )
 {
-	uint nFontBits;
-	QString chStr;
 	QFont *aRetFont = new QFont();
 
 	QStringList sl = aValue.split( QString::fromLatin1(","), QString::SkipEmptyParts );
-
 	if (sl.count() == 1) {
 		/* X11 font spec */
 		aRetFont->setRawMode( true );
 		aRetFont->setRawName( aValue );
-	} else if (sl.count() == 10) {
-		/* qt3 font spec */
+	} else {
 		aRetFont->fromString( aValue );
-	} else if (sl.count() == 6) {
-		/* backward compatible kde2 font spec */
-		aRetFont->setFamily( sl[0] );
-		aRetFont->setPointSize( sl[1].toInt() );
-		aRetFont->setWeight( sl[4].toUInt() );
-
-		aRetFont->setStyleHint( (QFont::StyleHint)sl[2].toUInt() );
-
-		nFontBits = sl[5].toUInt();
-		aRetFont->setItalic( (nFontBits & 0x01) != 0 );
-		aRetFont->setUnderline( (nFontBits & 0x02) != 0 );
-		aRetFont->setStrikeOut( (nFontBits & 0x04) != 0 );
-		aRetFont->setFixedPitch( (nFontBits & 0x08) != 0 );
-		aRetFont->setRawMode( (nFontBits & 0x20) != 0 );
 	}
 	aRetFont->setStyleStrategy( (QFont::StyleStrategy)
 	   (QFont::PreferMatch |
@@ -112,7 +88,14 @@ void initConfig( void )
 	_isReserve = _isLocal && getCfgInt( C_isReserve );
 	_hasConsole = _hasConsole && _isLocal && getCfgInt( C_hasConsole );
 	_authorized = getCfgInt( C_isAuthorized );
+	_grabInput =
+		(_grabInputPre == GRAB_NEVER) ? 0 :
+		(_grabInputPre == GRAB_ALWAYS) ? 1 :
+		!_authorized;
 
+	QByteArray dd = _dataDir.toUtf8();
+	if (access( dd.constData(), W_OK ))
+		logError( "Data directory %\"s not accessible: %m\n", dd.constData());
 	_stsFile = _dataDir + "/kdmsts";
 
 	// Greet String
@@ -141,7 +124,11 @@ void initConfig( void )
 			case 's': ptr = tuname.sysname; break;
 			case 'r': ptr = tuname.release; break;
 			case 'm': ptr = tuname.machine; break;
-			default: _greetString += i18n("[fix kdmrc]"); continue;
+			default: _greetString += i18nc("@item:intext substitution for "
+			                               "an undefined %X placeholder wrongly "
+			                               "given in the config file 'kdmrc', "
+			                               "telling the user to fix it",
+			                               "[fix kdmrc]"); continue;
 			}
 			_greetString += QString::fromLocal8Bit( ptr );
 		} else
