@@ -41,6 +41,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "kdecoration.h"
 #include "rules.h"
 #include "toplevel.h"
+#include "clientgroup.h"
 
 #ifdef HAVE_XSYNC
 #include <X11/extensions/sync.h>
@@ -54,6 +55,11 @@ class KStartupInfoData;
 
 namespace KWin
 {
+namespace TabBox {
+
+class TabBoxClientImpl;
+}
+
 
 class Workspace;
 class Client;
@@ -128,7 +134,10 @@ class Client
         QSize adjustedSize() const;
 
         QPixmap icon() const;
+        QPixmap icon( const QSize& size ) const;
         QPixmap miniIcon() const;
+        QPixmap bigIcon() const;
+        QPixmap hugeIcon() const;
 
         bool isActive() const;
         void setActive( bool );
@@ -218,6 +227,19 @@ class Client
         void resizeWithChecks( int w, int h, ForceGeometry_t force = NormalGeometrySet );
         void resizeWithChecks( const QSize& s, ForceGeometry_t force = NormalGeometrySet );
         void keepInArea( QRect area, bool partial = false );
+        void setElectricBorderMode( ElectricMaximizingMode mode );
+        ElectricMaximizingMode electricBorderMode() const;
+        void setElectricBorderMaximizing( bool maximizing );
+        bool isElectricBorderMaximizing() const;
+        QRect electricBorderMaximizeGeometry();
+
+        QSize sizeForClientSize( const QSize&, Sizemode mode = SizemodeAny, bool noframe = false ) const;
+
+        /** Set the quick tile mode ("snap") of this window.
+         * This will also handle preserving and restoring of window geometry as necessary.
+         * @param mode The tile mode (left/right) to give this window.
+         */
+        void setQuickTileMode( QuickTileMode mode, bool keyboard = false );
 
         void growHorizontal();
         void shrinkHorizontal();
@@ -228,6 +250,7 @@ class Client
         KShortcut shortcut() const;
         void setShortcut( const QString& cut );
 
+        WindowOperation mouseButtonToWindowOperation( Qt::MouseButtons button );
         bool performMouseCommand( Options::MouseCommand, const QPoint& globalPos, bool handled = false );
 
         QRect adjustedClientArea( const QRect& desktop, const QRect& area ) const;
@@ -270,7 +293,7 @@ class Client
 
         static bool belongToSameApplication( const Client* c1, const Client* c2, bool active_hack = false );
         static bool sameAppWindowRoleMatch( const Client* c1, const Client* c2, bool active_hack );
-        static void readIcons( Window win, QPixmap* icon, QPixmap* miniicon );
+        static void readIcons( Window win, QPixmap* icon, QPixmap* miniicon, QPixmap* bigicon, QPixmap* hugeicon );
 
         void minimize( bool avoid_animation = false );
         void unminimize( bool avoid_animation = false );
@@ -285,6 +308,24 @@ class Client
         StrutRect strutRect( StrutArea area ) const;
         StrutRects strutRects() const;
         bool hasStrut() const;
+
+        // Tabbing functions
+        ClientGroup* clientGroup() const; // Returns a pointer to client_group
+        void setClientGroup( ClientGroup* group );
+        /*
+        *   If shown is true the client is mapped and raised, if false
+        *   the client is unmapped and hidden, this function is called
+        *   when the tabbing group of the client switches its visible
+        *   client.
+        */
+        void setClientShown( bool shown );
+        /*
+        *   When a click is done in the decoration and it calls the group
+        *   to change the visible client it starts to move-resize the new
+        *   client, this function stops it.
+        */
+        void dontMoveResize();
+
         /**
          * Whether or not the window has a strut that expands through the invisible area of
          * an xinerama setup where the monitors are not the same resolution.
@@ -320,6 +361,10 @@ class Client
                         QRect(0, 0, width(), height());
         }
 
+        QRect transparentRect() const;
+
+        QRegion decorationPendingRegion() const;
+
         enum CoordinateMode {
             DecorationRelative, // Relative to the top left corner of the decoration
             WindowRelative      // Relative to the top left corner of the window
@@ -327,6 +372,8 @@ class Client
         void layoutDecorationRects(QRect &left, QRect &top, QRect &right, QRect &bottom, CoordinateMode mode) const;
         virtual void addRepaintFull();
 
+        TabBox::TabBoxClientImpl* tabBoxClient() const { return m_tabBoxClient; }
+
     private slots:
         void autoRaise();
         void shadeHover();
@@ -368,8 +415,10 @@ class Client
         bool buttonPressEvent( Window w, int button, int state, int x, int y, int x_root, int y_root );
         bool buttonReleaseEvent( Window w, int button, int state, int x, int y, int x_root, int y_root );
         bool motionNotifyEvent( Window w, int state, int x, int y, int x_root, int y_root );
+        void checkQuickTilingMaximizationZones( int xroot, int yroot );
 
-        void processDecorationButtonPress( int button, int state, int x, int y, int x_root, int y_root );
+        bool processDecorationButtonPress( int button, int state, int x, int y, int x_root, int y_root,
+            bool ignoreMenu = false );
 
     protected:
         virtual void debug( kdbgstream& stream ) const;
@@ -387,7 +436,6 @@ class Client
         void exportMappingState( int s ); // ICCCM 4.1.3.1, 4.1.4, NETWM 2.5.1
         bool isManaged() const; ///< Returns false if this client is not yet managed
         void updateAllowedActions( bool force = false );
-        QSize sizeForClientSize( const QSize&, Sizemode mode = SizemodeAny, bool noframe = false ) const;
         QRect fullscreenMonitorsArea( NETFullscreenMonitors topology ) const;
         void changeMaximize( bool horizontal, bool vertical, bool adjust );
         void checkMaximizeGeometry();
@@ -480,6 +528,12 @@ class Client
             Kept ///< The frame should be unmapped, but is kept (For compositing)
             };
         MappingState mapping_state;
+
+        /** The quick tile mode of this window.
+         */
+        QuickTileMode quick_tile_mode;
+        QRect geom_pretile;
+
         void readTransient();
         Window verifyTransientFor( Window transient_for, bool set );
         void addTransient( Client* cl );
@@ -513,7 +567,8 @@ class Client
         uint hidden : 1; ///< Forcibly hidden by calling hide()
         uint modal : 1; ///< NET::Modal
         uint noborder : 1;
-        uint app_noborder : 1; ///< The app requested no border using something (window type, motif hints)
+        uint app_noborder : 1; ///< App requested no border via window type, shape extension, etc.
+        uint motif_noborder : 1; ///< App requested no border via Motif WM hints
         uint urgency : 1; ///< XWMHints, UrgencyHint
         uint ignore_focus_stealing : 1; ///< Don't apply focus stealing prevention to this client
         uint demands_attention : 1;
@@ -523,6 +578,8 @@ class Client
         void getWindowProtocols();
         QPixmap icon_pix;
         QPixmap miniicon_pix;
+        QPixmap bigicon_pix;
+        QPixmap hugeicon_pix;
         QCursor cursor;
         // DON'T reorder - Saved to config files !!!
         enum FullScreenMode
@@ -543,6 +600,7 @@ class Client
         QString cap_normal, cap_iconic, cap_suffix;
         Group* in_group;
         Window window_group;
+        ClientGroup* client_group;
         Layer in_layer;
         QTimer* ping_timer;
         QProcess* process_killer;
@@ -581,6 +639,10 @@ class Client
         QTimer* demandAttentionKNotifyTimer;
         QPixmap decorationPixmapLeft, decorationPixmapRight, decorationPixmapTop, decorationPixmapBottom;
         PaintRedirector* paintRedirector;
+        TabBox::TabBoxClientImpl* m_tabBoxClient;
+
+        bool electricMaximizing;
+        ElectricMaximizingMode electricMode;
 
         friend bool performTransiencyCheck();
     };
@@ -672,6 +734,16 @@ inline Group* Client::group()
     return in_group;
     }
 
+inline void Client::setClientGroup( ClientGroup* group )
+    {
+    client_group = group;
+    }
+
+inline ClientGroup* Client::clientGroup() const
+    {
+    return client_group;
+    }
+
 inline bool Client::isMinimized() const
     {
     return minimized;
@@ -684,7 +756,8 @@ inline bool Client::isActive() const
 
 inline bool Client::isShown( bool shaded_is_shown ) const
     {
-    return !isMinimized() && ( !isShade() || shaded_is_shown ) && !hidden;
+    return !isMinimized() && ( !isShade() || shaded_is_shown ) && !hidden &&
+        ( clientGroup() == NULL || clientGroup()->visible() == this );
     }
 
 inline bool Client::isHiddenInternal() const
@@ -712,6 +785,16 @@ inline QPixmap Client::miniIcon() const
     return miniicon_pix;
     }
 
+inline QPixmap Client::bigIcon() const
+    {
+    return bigicon_pix;
+    }
+
+inline QPixmap Client::hugeIcon() const
+    {
+    return hugeicon_pix;
+    }
+
 inline QRect Client::geometryRestore() const
     {
     return geom_restore;
