@@ -48,9 +48,11 @@ namespace KWin
     rule_##var->setWhatsThis( type##RuleDesc );
 
 RulesWidget::RulesWidget( QWidget* parent )
-    : RulesWidgetBase( parent )
-    , detect_dlg( NULL )
+    : detect_dlg( NULL )
     {
+    Q_UNUSED( parent );
+    setupUi(this);
+
     QString enableDesc =
         i18n( "Enable this checkbox to alter this window property for the specified window(s)." );
     QString setRuleDesc =
@@ -96,6 +98,9 @@ RulesWidget::RulesWidget( QWidget* parent )
     SETUP( skippager, set );
     SETUP( acceptfocus, force );
     SETUP( closeable, force );
+    SETUP( autogroup, force );
+    SETUP( autogroupfg, force );
+    SETUP( autogroupid, force );
     SETUP( opacityactive, force );
     SETUP( opacityinactive, force );
     SETUP( shortcut, force );
@@ -122,7 +127,7 @@ RulesWidget::RulesWidget( QWidget* parent )
 void RulesWidget::updateEnable##var() \
     { \
     /* leave the label readable label_##var->setEnabled( enable_##var->isChecked() && rule_##var->currentIndex() != 0 );*/ \
-    Ui_RulesWidgetBase::var->setEnabled( enable_##var->isChecked() && rule_##var->currentIndex() != 0 ); \
+    Ui::RulesWidgetBase::var->setEnabled( enable_##var->isChecked() && rule_##var->currentIndex() != 0 ); \
     }
 
 // geometry tab
@@ -143,6 +148,9 @@ UPDATE_ENABLE_SLOT( skiptaskbar )
 UPDATE_ENABLE_SLOT( skippager )
 UPDATE_ENABLE_SLOT( acceptfocus )
 UPDATE_ENABLE_SLOT( closeable )
+UPDATE_ENABLE_SLOT( autogroup )
+UPDATE_ENABLE_SLOT( autogroupfg )
+UPDATE_ENABLE_SLOT( autogroupid )
 UPDATE_ENABLE_SLOT( opacityactive )
 UPDATE_ENABLE_SLOT( opacityinactive )
 void RulesWidget::updateEnableshortcut()
@@ -352,14 +360,14 @@ static NET::WindowType comboToType( int val )
         { \
         enable_##var->setChecked( false ); \
         rule_##var->setCurrentIndex( 0 ); \
-        Ui_RulesWidgetBase::var->uimethod0; \
+        Ui::RulesWidgetBase::var->uimethod0;    \
         updateEnable##var(); \
         } \
     else \
         { \
         enable_##var->setChecked( true ); \
         rule_##var->setCurrentIndex( type##_rule_to_combo[ rules->var##rule ] ); \
-        Ui_RulesWidgetBase::var->uimethod( func( rules->var )); \
+        Ui::RulesWidgetBase::var->uimethod( func( rules->var )); \
         updateEnable##var(); \
         }
 
@@ -418,6 +426,9 @@ void RulesWidget::setRules( Rules* rules )
     CHECKBOX_SET_RULE( skippager, );
     CHECKBOX_FORCE_RULE( acceptfocus, );
     CHECKBOX_FORCE_RULE( closeable, );
+    CHECKBOX_FORCE_RULE( autogroup, );
+    CHECKBOX_FORCE_RULE( autogroupfg, );
+    LINEEDIT_FORCE_RULE( autogroupid, );
     LINEEDIT_FORCE_RULE( opacityactive, intToStr );
     LINEEDIT_FORCE_RULE( opacityinactive, intToStr );
     LINEEDIT_SET_RULE( shortcut, );
@@ -443,7 +454,7 @@ void RulesWidget::setRules( Rules* rules )
     if( enable_##var->isChecked() && rule_##var->currentIndex() >= 0) \
         { \
         rules->var##rule = combo_to_##type##_rule[ rule_##var->currentIndex() ]; \
-        rules->var = func( Ui_RulesWidgetBase::var->uimethod()); \
+        rules->var = func( Ui::RulesWidgetBase::var->uimethod()); \
         } \
     else \
         rules->var##rule = Rules::Unused##Type##Rule;
@@ -475,16 +486,16 @@ Rules* RulesWidget::rules() const
         rules->types = NET::AllTypesMask;
     else
         {
-        rules->types |= types->item(0)->isSelected() ? NET::NormalMask : 0;
-        rules->types |= types->item(1)->isSelected() ? NET::DialogMask : 0;
-        rules->types |= types->item(2)->isSelected() ? NET::UtilityMask : 0;
-        rules->types |= types->item(3)->isSelected() ? NET::DockMask : 0;
-        rules->types |= types->item(4)->isSelected() ? NET::ToolbarMask : 0;
-        rules->types |= types->item(5)->isSelected() ? NET::MenuMask : 0;
-        rules->types |= types->item(6)->isSelected() ? NET::SplashMask : 0;
-        rules->types |= types->item(7)->isSelected() ? NET::DesktopMask : 0;
-        rules->types |= types->item(8)->isSelected() ? NET::OverrideMask : 0;
-        rules->types |= types->item(9)->isSelected() ? NET::TopMenuMask : 0;
+        rules->types |= types->item(0)->isSelected() ? NET::NormalMask : 0U;
+        rules->types |= types->item(1)->isSelected() ? NET::DialogMask : 0U;
+        rules->types |= types->item(2)->isSelected() ? NET::UtilityMask : 0U;
+        rules->types |= types->item(3)->isSelected() ? NET::DockMask : 0U;
+        rules->types |= types->item(4)->isSelected() ? NET::ToolbarMask : 0U;
+        rules->types |= types->item(5)->isSelected() ? NET::MenuMask : 0U;
+        rules->types |= types->item(6)->isSelected() ? NET::SplashMask : 0U;
+        rules->types |= types->item(7)->isSelected() ? NET::DesktopMask : 0U;
+        rules->types |= types->item(8)->isSelected() ? NET::OverrideMask : 0U;
+        rules->types |= types->item(9)->isSelected() ? NET::TopMenuMask : 0U;
         }
     rules->title = title->text();
     rules->titlematch = static_cast< Rules::StringMatch >( title_match->currentIndex());
@@ -508,6 +519,9 @@ Rules* RulesWidget::rules() const
     CHECKBOX_SET_RULE( skippager, );
     CHECKBOX_FORCE_RULE( acceptfocus, );
     CHECKBOX_FORCE_RULE( closeable, );
+    CHECKBOX_FORCE_RULE( autogroup, );
+    CHECKBOX_FORCE_RULE( autogroupfg, );
+    LINEEDIT_FORCE_RULE( autogroupid, );
     LINEEDIT_FORCE_RULE( opacityactive, strToInt );
     LINEEDIT_FORCE_RULE( opacityinactive, strToInt );
     LINEEDIT_SET_RULE( shortcut, );
@@ -599,7 +613,7 @@ void RulesWidget::detected( bool ok )
 #define GENERIC_PREFILL( var, func, info, uimethod ) \
     if( !enable_##var->isChecked()) \
         { \
-        Ui_RulesWidgetBase::var->uimethod( func( info )); \
+        Ui::RulesWidgetBase::var->uimethod( func( info )); \
         }
 
 #define CHECKBOX_PREFILL( var, func, info ) GENERIC_PREFILL( var, func, info, setChecked )
@@ -625,6 +639,9 @@ void RulesWidget::prefillUnusedValues( const KWindowInfo& info )
     CHECKBOX_PREFILL( skippager,, info.state() & NET::SkipPager );
     //CHECKBOX_PREFILL( acceptfocus, );
     //CHECKBOX_PREFILL( closeable, );
+    //CHECKBOX_PREFILL( autogroup, );
+    //CHECKBOX_PREFILL( autogroupfg, );
+    //LINEEDIT_PREFILL( autogroupid, );
     LINEEDIT_PREFILL( opacityactive, intToStr, 100 /*get the actual opacity somehow*/);
     LINEEDIT_PREFILL( opacityinactive, intToStr, 100 /*get the actual opacity somehow*/);
     //LINEEDIT_PREFILL( shortcut, );
@@ -660,7 +677,7 @@ bool RulesWidget::finalCheck()
             all_types = false;
     if( wmclass_match->currentIndex() == Rules::UnimportantMatch && all_types )
         {
-        if( KMessageBox::warningContinueCancel( topLevelWidget(),
+        if( KMessageBox::warningContinueCancel( window(),
             i18n( "You have specified the window class as unimportant.\n"
                   "This means the settings will possibly apply to windows from all applications. "
                   "If you really want to create a generic setting, it is recommended you at least "
@@ -679,10 +696,11 @@ void RulesWidget::prepareWindowSpecific( WId window )
 
 void RulesWidget::shortcutEditClicked()
     {
-    EditShortcutDialog dlg( topLevelWidget());
-    dlg.setShortcut( shortcut->text());
-    if( dlg.exec() == QDialog::Accepted )
-        shortcut->setText( dlg.shortcut());
+    QPointer<EditShortcutDialog> dlg = new EditShortcutDialog( window());
+    dlg->setShortcut( shortcut->text());
+    if( dlg->exec() == QDialog::Accepted )
+        shortcut->setText( dlg->shortcut());
+    delete dlg;
     }
 
 RulesDialog::RulesDialog( QWidget* parent, const char* name )
@@ -692,6 +710,7 @@ RulesDialog::RulesDialog( QWidget* parent, const char* name )
       setModal( true );
       setCaption( i18n( "Edit Window-Specific Settings" ) );
       setButtons( Ok | Cancel );
+      setWindowIcon(KIcon("preferences-system-windows-actions"));
 
       widget = new RulesWidget( this );
       setMainWidget( widget );
@@ -740,9 +759,10 @@ EditShortcut::EditShortcut( QWidget* parent )
 
 void EditShortcut::editShortcut()
     {
-    ShortcutDialog dlg( QKeySequence( shortcut->text()), topLevelWidget());
-    if( dlg.exec() == QDialog::Accepted )
-        shortcut->setText( dlg.shortcut().toString());
+    QPointer< ShortcutDialog > dlg = new ShortcutDialog( QKeySequence( shortcut->text()), window());
+    if( dlg->exec() == QDialog::Accepted )
+        shortcut->setText( dlg->shortcut().toString());
+    delete dlg;
     }
 
 void EditShortcut::clearShortcut()
@@ -777,6 +797,8 @@ ShortcutDialog::ShortcutDialog( const QKeySequence& cut, QWidget* parent )
     , widget( new KKeySequenceWidget( this ))
     {
     widget->setKeySequence( cut );
+    // It's a global shortcut so don't allow multikey shortcuts
+    widget->setMultiKeyShortcutsAllowed(false);
     setMainWidget( widget );
     }
 
