@@ -54,6 +54,10 @@ class SceneXrender
     private:
         void paintTransformedScreen( int mask );
         void createBuffer();
+        void flushBuffer( int mask, QRegion damage );
+        bool selfCheck();
+        void selfCheckSetup();
+        bool selfCheckFinish();
         XRenderPictFormat* format;
         Picture front;
         static Picture buffer;
@@ -77,11 +81,15 @@ class SceneXrender::Window
     private:
         Picture picture();
         Picture alphaMask( double opacity );
+        QRect mapToScreen(int mask, const WindowPaintData &data, const QRect &rect) const;
+        QPoint mapToScreen(int mask, const WindowPaintData &data, const QPoint &point) const;
+        void prepareTempPixmap(const QPixmap *left, const QPixmap *top, const QPixmap *right, const QPixmap *bottom);
         Picture _picture;
         XRenderPictFormat* format;
         Picture alpha;
         double alpha_cached_opacity;
         QRegion transformed_shape;
+        static QPixmap *temp_pixmap;
     };
 
 inline
