@@ -68,6 +68,9 @@ ProcessModelPrivate::ProcessModelPrivate() :  mBlankPixmap(HEADING_X_ICON_SIZE,1
     mShowingTooltips = true;
     mNormalizeCPUUsage = true;
     mIoInformation = ProcessModel::ActualBytes;
+    mMovingRow = false;
+    mRemovingRow = false;
+    mInsertingRow = false;
 }
 
 ProcessModelPrivate::~ProcessModelPrivate()
@@ -269,7 +272,6 @@ void ProcessModelPrivate::windowRemoved(WId wid) {
     WindowInfo *window = mWIdToWindowInfo.take(wid);
     if(!window) return;
     qlonglong pid = window->pid;
-    delete window;
 
     QMultiHash<qlonglong, WindowInfo*>::iterator i = mPidToWindowInfo.find(pid);
     while (i != mPidToWindowInfo.end() && i.key() == pid) {
@@ -279,6 +281,7 @@ void ProcessModelPrivate::windowRemoved(WId wid) {
         } else
             i++;
     }
+    delete window;
 
     //Update the model so that it redraws and resorts
     KSysGuard::Process *process = mProcesses->getProcess(pid);
@@ -592,6 +595,11 @@ void ProcessModelPrivate::processChanged(KSysGuard::Process *process, bool onlyT
 void ProcessModelPrivate::beginInsertRow( KSysGuard::Process *process)
 {
     Q_ASSERT(process);
+    Q_ASSERT(!mRemovingRow);
+    Q_ASSERT(!mInsertingRow);
+    Q_ASSERT(!mMovingRow);
+    mInsertingRow = true;
+
     if(mSimple) {
         int row = mProcesses->processCount();
         q->beginInsertRows( QModelIndex(), row, row );
@@ -606,12 +614,21 @@ void ProcessModelPrivate::beginInsertRow( KSysGuard::Process *process)
     q->beginInsertRows(parentModelIndex, row, row);
 }
 void ProcessModelPrivate::endInsertRow() {
+    Q_ASSERT(!mRemovingRow);
+    Q_ASSERT(mInsertingRow);
+    Q_ASSERT(!mMovingRow);
+    mInsertingRow = false;
+
     q->endInsertRows();
 }
 void ProcessModelPrivate::beginRemoveRow( KSysGuard::Process *process )
 {
     Q_ASSERT(process);
     Q_ASSERT(process->pid > 0);
+    Q_ASSERT(!mRemovingRow);
+    Q_ASSERT(!mInsertingRow);
+    Q_ASSERT(!mMovingRow);
+    mRemovingRow = true;
 
     if(mSimple) {
         return q->beginRemoveRows(QModelIndex(), process->index, process->index);
@@ -619,6 +636,7 @@ void ProcessModelPrivate::beginRemoveRow( KSysGuard::Process *process )
         int row = process->parent->children.indexOf(process);
         if(row == -1) {
             kDebug(1215) << "A serious problem occurred in remove row.";
+            mRemovingRow = false;
             return;
         }
 
@@ -627,31 +645,42 @@ void ProcessModelPrivate::beginRemoveRow( KSysGuard::Process *process )
 }
 void ProcessModelPrivate::endRemoveRow()
 {
+    Q_ASSERT(!mInsertingRow);
+    Q_ASSERT(!mMovingRow);
+    if(!mRemovingRow)
+        return;
+    mRemovingRow = false;
+
     q->endRemoveRows();
 }
 
 
 void ProcessModelPrivate::beginMoveProcess(KSysGuard::Process *process, KSysGuard::Process *new_parent)
 {
+    Q_ASSERT(!mRemovingRow);
+    Q_ASSERT(!mInsertingRow);
+    Q_ASSERT(!mMovingRow);
+
     if(mSimple) return;  //We don't need to move processes when in simple mode
-    emit q->layoutAboutToBeChanged ();
+    mMovingRow = true;
 
-    //FIXME
     int current_row = process->parent->children.indexOf(process);
-    int new_row = new_parent->children.count();
     Q_ASSERT(current_row != -1);
-
-    QList<QModelIndex> fromIndexes;
-    QList<QModelIndex> toIndexes;
-    for(int i=0; i < q->columnCount(); i++) {
-        fromIndexes << q->createIndex(current_row, i, process);
-        toIndexes << q->createIndex(new_row, i, process);
-    }
-    q->changePersistentIndexList(fromIndexes, toIndexes);
+    int new_row = new_parent->children.count();
+    QModelIndex sourceParent = q->getQModelIndex( process->parent, 0);
+    QModelIndex destinationParent = q->getQModelIndex( new_parent, 0 );
+    mMovingRow = q->beginMoveRows(sourceParent, current_row, current_row, destinationParent, new_row);
+    Q_ASSERT(mMovingRow);
 }
 void ProcessModelPrivate::endMoveRow()
 {
-    emit q->layoutChanged();
+    Q_ASSERT(!mInsertingRow);
+    Q_ASSERT(!mRemovingRow);
+    if(!mMovingRow)
+        return;
+    mMovingRow = false;
+
+    q->endMoveRows();
 }
 
 
